--[[************************************************************************************************************************************************************
	
	MobileBeehive
	
	09/2018 © Saty MODS (SatyMODS / http://forum.lsczech.cz)
	
****************************************************************************************************************************************************************
	
	(EN) INFO
	
	Free for use on mods. Modifications only with my permission !
	
****************************************************************************************************************************************************************
	
	(CZ) INFO
	
	Pro volne pouziti v modech. ZAKAZ modifikaci bez meho souhlasu !
	
************************************************************************************************************************************************************]]--

source(g_currentModDirectory .. "SCR/overrideScripts.lua");

MobileBeehive = {};

function MobileBeehive.prerequisitesPresent(specializations)
	return true;
end;

function MobileBeehive:preLoad(savegame)
    self.loadTestAreaFromXML = MobileBeehive.loadTestAreaFromXML;
end;

function MobileBeehive:load(savegame)
	self.setBeesActive = MobileBeehive.setBeesActive;

	self.beesActive = false;
	self.beehiveIsNotAttached = true;
	
    self.particleSystem = {};
    ParticleUtil.loadParticleSystem(self.xmlFile, self.particleSystem, "vehicle.beehive.particleSystem", self.components[1].node, true, nil, self.baseDirectory);
	
    if self.isClient then
        self.sampleIdle = SoundUtil.loadSample(self.xmlFile, {}, "vehicle.beehive.idleSound", nil, self.baseDirectory, self.components[1].node);
		
        self.testAreas = {}
        local i = 0
        while true do
            local key = string.format("vehicle.beehive.testAreas.testArea(%d)", i);
            if not hasXMLProperty(self.xmlFile, key) then
                break;
            end;
			
            local testArea = {};
            if self:loadTestAreaFromXML(testArea, self.xmlFile, key) then
                table.insert(self.testAreas, testArea);
            end;
            i = i + 1;
        end;
    end;

    local incomePerHour = getXMLFloat(self.xmlFile, "vehicle.beehive.incomePerActiveHour" .. g_currentMission.missionInfo.difficulty);
	self.incomePerMinute = incomePerHour / 60;
	
	self.incomeInMinutes  = 0;
	self.checkTimeMinutes = nil;
	
	self:setBeesActive(false);
		
    if savegame ~= nil then
		self.incomeInMinutes = Utils.getNoNil(getXMLFloat(savegame.xmlFile, savegame.key .. ".MobileBeehive#activeMinutes"), 0);
	end;
end;

function MobileBeehive:postLoad(savegame)
end;

function MobileBeehive:getSaveAttributesAndNodes(nodeIdent)
	local nodes = nodeIdent .. '<MobileBeehive activeMinutes="' .. tostring(self.incomeInMinutes) .. '" />';
	return nil, nodes;
end;

function MobileBeehive:update(dt)
	local beesActive = true;
	local curtime = g_currentMission.environment.dayTime/(1000*60*60);
	local timeHours = math.floor(curtime);
	local timeMinutes = math.floor((curtime-timeHours)*60);
	
	local incomeInMinutesCoef = 1;
	local fruitFind = {};
	
	fruitFind[FruitUtil.FRUITTYPE_GRASS] = 0;
	fruitFind[FruitUtil.FRUITTYPE_RAPE] = 0;
	fruitFind[FruitUtil.FRUITTYPE_SUNFLOWER] = 0;
	
	if g_currentMission.environment.currentRain ~= nil or curtime < 8 or curtime > 18 then
		beesActive = false;
	end;
	
	if not self.beehiveIsNotAttached then
		beesActive = false;
	end;
	
	if g_seasons ~= nil then
		if g_seasons.environment:currentSeason() == 3 then
			beesActive = false;
		end;
	end;
	
    if self.isClient then
		if self.beesActive ~= beesActive then
			self:setBeesActive(beesActive);
		end;
		
		if self.beesActive or self:getIsActiveForInput() then
			for _, testArea in pairs(self.testAreas) do
				local x ,_,z  = getWorldTranslation(testArea.start);
				local x1,_,z1 = getWorldTranslation(testArea.width);
				local x2,_,z2 = getWorldTranslation(testArea.height);
				local fruitId = MobileBeehive.getFruitIdArea(x, z, x1, z1, x2, z2);
				
				if fruitId ~= 0 then
					fruitFind[fruitId] = fruitFind[fruitId] + 1;
				end;
			end;
			
			local grass = fruitFind[FruitUtil.FRUITTYPE_GRASS];
			local rape = fruitFind[FruitUtil.FRUITTYPE_RAPE];
			local sunFlower = fruitFind[FruitUtil.FRUITTYPE_SUNFLOWER];
			
			--print("Grass (" .. tostring(grass) .. ") * Rape (" .. tostring(rape) .. ") * SunFlower (" .. tostring(sunFlower) .. ")");
			
			if sunFlower > 0 and sunFlower >= rape then
				incomeInMinutesCoef = 1.8;
			elseif rape > 0 and rape > sunFlower then
				incomeInMinutesCoef = 1.5;
			elseif grass > 8 then
				incomeInMinutesCoef = 1.2;
			end;
		end;
		
		if self:getIsActiveForInput() then
			if incomeInMinutesCoef > 1 then
				g_currentMission:addExtraPrintText(g_i18n:getText("beehive_production") .. " BONUS +" .. tostring((incomeInMinutesCoef-1)*100) .. "%");
			end;
		end;
	end;
	
	if self.isServer then
		if timeMinutes ~= self.checkTimeMinutes then
			self.checkTimeMinutes = timeMinutes;
			if self.beesActive then
				self.incomeInMinutes = self.incomeInMinutes + incomeInMinutesCoef;
			end;
		end;
		if curtime > 18 and self.incomeInMinutes > 0 then
			local incomeMoney = math.floor(self.incomeInMinutes * self.incomePerMinute);
			
			g_currentMission:addSharedMoney(incomeMoney, "propertyIncome");
			g_currentMission:addMoneyChange(incomeMoney, EconomyManager.MONEY_TYPE_PROPERTY_INCOME);
			
			self.incomeInMinutes  = 0;
		end;
	end;
end;

function MobileBeehive:updateTick(dt)
end;

function MobileBeehive:setBeesActive(state)
	if state then
		SoundUtil.play3DSample(self.sampleIdle)
		Utils.setEmittingState(self.particleSystem, true);
	else
		SoundUtil.stop3DSample(self.sampleIdle)
		Utils.setEmittingState(self.particleSystem, false);
	end;
	
	self.beesActive = state;
end;

function MobileBeehive:loadTestAreaFromXML(testArea, xmlFile, key)
    local start = Utils.indexToObject(self.components, getXMLString(xmlFile, key .. "#startIndex"));
    local width = Utils.indexToObject(self.components, getXMLString(xmlFile, key .. "#widthIndex"));
    local height = Utils.indexToObject(self.components, getXMLString(xmlFile, key .. "#heightIndex"));

    if start ~= nil and width ~= nil and height ~= nil then
        testArea.start = start;
        testArea.width = width;
        testArea.height = height;
        return true;
    end;
	
    return false;
end

function MobileBeehive:readStream(streamId, connection)
end;

function MobileBeehive:writeStream(streamId, connection)
end;

function MobileBeehive:delete()
    if self.particleSystem ~= nil then
        ParticleUtil.deleteParticleSystem(self.particleSystem);
    end;

    if self.sampleIdle ~= nil then
        SoundUtil.deleteSample(self.sampleIdle);
    end;
end;

function MobileBeehive:onAttach(attacherVehicle)
	self.beehiveIsNotAttached = false;
end;

function MobileBeehive:onDetach()
	self.beehiveIsNotAttached = true;
end;

function MobileBeehive:draw()
end;

function MobileBeehive:mouseEvent(posX, posY, isDown, isUp, button)
end;

function MobileBeehive:keyEvent(unicode, sym, modifier, isDown)
end;


function MobileBeehive.getFruitIdArea(startWorldX, startWorldZ, widthWorldX, widthWorldZ, heightWorldX, heightWorldZ)
	local grassId 		= FruitUtil.FRUITTYPE_GRASS;
	local rapeId  		= FruitUtil.FRUITTYPE_RAPE;
	local sunFlowerId  	= FruitUtil.FRUITTYPE_SUNFLOWER;
	local fruitDensity 	= {};
	
	for i = 1, FruitUtil.NUM_FRUITTYPES do
		if i == grassId or i == rapeId or i == sunFlowerId then
			local ids = g_currentMission.fruits[i];
			if ids == nil or ids.id == 0 then
				break;
			end;

			local id = ids.id;
			local desc = FruitUtil.fruitIndexToDesc[i];

			local minState, maxState = 0, 0; 
			
			if i == grassId then
				minState, maxState = 3, 3;
			elseif i == rapeId then
				minState, maxState = 3, 3;
			elseif i == sunFlowerId then
				minState, maxState = 3, 3;
			end;

			setDensityReturnValueShift(id, -1);
			setDensityCompareParams(id, "between", minState+1, maxState+1);
			
			local x, z, widthX, widthZ, heightX, heightZ = Utils.getXZWidthAndHeight(nil, startWorldX, startWorldZ, widthWorldX, widthWorldZ, heightWorldX, heightWorldZ);
			local density, _ = getDensityParallelogram(id, x, z, widthX, widthZ, heightX, heightZ, 0, g_currentMission.numFruitStateChannels);

			fruitDensity[i] = density;
			
			setDensityCompareParams(id, "greater", -1);
			setDensityReturnValueShift(id, 0);
		end;
	end;
	
	if fruitDensity[sunFlowerId] > 0 then
		return sunFlowerId;
	elseif fruitDensity[rapeId] > 0 then
		return rapeId;
	elseif fruitDensity[grassId] > 0 then
		return grassId;
	else
		return 0;
	end;
end;
